function q_out = qMultiTelescopeZ(q_in, Radius, Pos, z)
% function q_out = qMultiTelescopeZ(q_in, Radius, Pos, z)
%
% calculates a beam from anywhere,  assuming a q_in at the z = 0 and
% optics of ROC Radius positioned at Pos with a beam traveling towards
% positive z.  The optics can be placed at negative z.

% The number of optics
nOpt = length(Radius);
% each z position gets its own ray matrix
M = repmat(eye(2,2),[1 1 length(z)]);

%% First we define the starting z as the minimum of z and the mirrors and
% set up the q accordingly.

zmin = min([z Pos]);  % here is where we start calculating
q_min = q_in + zmin;   % Set up the initial q to be at the beginning of 
                      % the calculation. 
%Pos = Pos - zmin;  % Now all mirror positions are relative to the zmin


%% Do the forward propagation
% setup the length limits vector
limVec = [zmin Pos(1); 
          Pos(1:end-1)' Pos(2:end)';
          Pos(end) inf];

thisR = [Radius, inf];

for ii = 1:size(limVec,1)
    idx = find(z>limVec(ii,1) & z < limVec(ii,2));
    if ~isempty(idx)
        for jj = idx
             M(:,:,jj) = [1 z(jj)-limVec(ii,1); 0 1] * M(:,:,jj);
        end
    end
    
    idx = find(z>=limVec(ii,2));
    if ~isempty(idx)
        dL = limVec(ii,2) - limVec(ii,1);
        for jj = idx
            M(:,:,jj) = [1 0; -2/thisR(ii) 1]* [1 dL; 0 1] * M(:,:,jj);
        end
    end
end

if 0
%% Do the backwards propagation
% setup the length limits vector
limVec = [-inf min(Pos(1),0) ;
    min(Pos(1:end-1)',0) min(Pos(2:end)',0);
    min(0,Pos(end)) 0]
limVec(find(limVec(:,1) == limVec(:,2)),:) = [];
thisR = [inf, Radius];

for ii = size(limVec,1):-1:1
    idx = find(z<=0 & z>limVec(ii,1) & z < limVec(ii,2));
    if ~isempty(idx)
        for jj = idx
            M(:,:,jj) = [1 z(jj)-limVec(ii,2); 0 1] * M(:,:,jj);
        end
    end
    idx = find(z<=0 & z<=limVec(ii,1));
    if ~isempty(idx)
        dL = limVec(ii,1) - limVec(ii,2);
        for jj = idx
            
            M(:,:,jj) = [1 0; 2/thisR(ii) 1]* [1 dL; 0 1] * M(:,:,jj);
        end
    end
end
end


for jj = 1:length(z)
    q_out(jj) = (M(1,1,jj)*q_min + M(1,2,jj) )  ./ (M(2,1,jj)*q_min + M(2,2,jj));
    
end
